@@ -17,40 +17,42 @@
 
 #include <gtest/gtest.h>
 
-#include <ignition/plugin/Loader.hh>
+#include <gz/plugin/Loader.hh>
 
 #include <ignition/common/ImageHeightmap.hh>
-#include <ignition/common/MeshManager.hh>
+#include <gz/common/MeshManager.hh>
 #include <ignition/common/Filesystem.hh>
 
-#include <ignition/math/eigen3/Conversions.hh>
+#include <gz/math/eigen3/Conversions.hh>
 
-#include <ignition/physics/Joint.hh>
-#include <ignition/physics/RequestEngine.hh>
-#include <ignition/physics/RevoluteJoint.hh>
+#include <gz/physics/Joint.hh>
+#include <gz/physics/RequestEngine.hh>
+#include <gz/physics/RevoluteJoint.hh>
 
 #include "EntityManagementFeatures.hh"
 #include "JointFeatures.hh"
 #include "KinematicsFeatures.hh"
 #include "ShapeFeatures.hh"
 
-struct TestFeatureList : ignition::physics::FeatureList<
-    ignition::physics::dartsim::EntityManagementFeatureList,
-    ignition::physics::dartsim::JointFeatureList,
-    ignition::physics::dartsim::KinematicsFeatureList,
-    ignition::physics::dartsim::ShapeFeatureList
+using namespace ignition;
+
+struct TestFeatureList : physics::FeatureList<
+    physics::dartsim::EntityManagementFeatureList,
+    physics::dartsim::JointFeatureList,
+    physics::dartsim::KinematicsFeatureList,
+    physics::dartsim::ShapeFeatureList
 > { };
 
 TEST(EntityManagement_TEST, ConstructEmptyWorld)
 {
-  ignition::plugin::Loader loader;
+  plugin::Loader loader;
   loader.LoadLib(dartsim_plugin_LIB);
 
-  ignition::plugin::PluginPtr dartsim =
+  plugin::PluginPtr dartsim =
       loader.Instantiate("ignition::physics::dartsim::Plugin");
 
   auto engine =
-      ignition::physics::RequestEngine3d<TestFeatureList>::From(dartsim);
+      physics::RequestEngine3d<TestFeatureList>::From(dartsim);
   ASSERT_NE(nullptr, engine);
 
   auto world = engine->ConstructEmptyWorld("empty world");
@@ -117,7 +119,7 @@ TEST(EntityManagement_TEST, ConstructEmptyWorld)
   prismatic->SetVelocity(0, zVel);
   prismatic->SetAcceleration(0, zAcc);
 
-  const ignition::physics::FrameData3d childData =
+  const physics::FrameData3d childData =
       child->FrameDataRelativeToWorld();
 
   const Eigen::Vector3d childPosition = childData.pose.translation();
@@ -140,7 +142,7 @@ TEST(EntityManagement_TEST, ConstructEmptyWorld)
   childSpherePose.translate(Eigen::Vector3d(0.0, yPos, 0.0));
   auto sphere = child->AttachSphereShape("child sphere", 1.0, childSpherePose);
 
-  const ignition::physics::FrameData3d sphereData =
+  const physics::FrameData3d sphereData =
       sphere->FrameDataRelativeToWorld();
 
   const Eigen::Vector3d spherePosition = sphereData.pose.translation();
@@ -158,7 +160,7 @@ TEST(EntityManagement_TEST, ConstructEmptyWorld)
   EXPECT_DOUBLE_EQ(0.0, sphereAcceleration.y());
   EXPECT_DOUBLE_EQ(zAcc, sphereAcceleration.z());
 
-  const ignition::physics::FrameData3d relativeSphereData =
+  const physics::FrameData3d relativeSphereData =
       sphere->FrameDataRelativeTo(*child);
   const Eigen::Vector3d relativeSpherePosition =
       relativeSphereData.pose.translation();
@@ -171,7 +173,7 @@ TEST(EntityManagement_TEST, ConstructEmptyWorld)
 
   const std::string meshFilename = ignition::common::joinPaths(
       IGNITION_PHYSICS_RESOURCE_DIR, "chassis.dae");
-  auto &meshManager = *ignition::common::MeshManager::Instance();
+  auto &meshManager = *common::MeshManager::Instance();
   auto *mesh = meshManager.Load(meshFilename);
 
   auto meshShape = meshLink->AttachMeshShape("chassis", *mesh);
@@ -188,11 +190,11 @@ TEST(EntityManagement_TEST, ConstructEmptyWorld)
   EXPECT_NEAR(meshShapeSize[1], 0.3831, 1e-4);
   EXPECT_NEAR(meshShapeSize[2], 0.1956, 1e-4);
 
-  const ignition::math::Pose3d pose(0, 0, 0.2, 0, 0, 0);
-  const ignition::math::Vector3d scale(0.5, 1.0, 0.25);
+  const math::Pose3d pose(0, 0, 0.2, 0, 0, 0);
+  const math::Vector3d scale(0.5, 1.0, 0.25);
   auto meshShapeScaled = meshLink->AttachMeshShape("small_chassis", *mesh,
-                          ignition::math::eigen3::convert(pose),
-                          ignition::math::eigen3::convert(scale));
+                          math::eigen3::convert(pose),
+                          math::eigen3::convert(scale));
   const auto meshShapeScaledSize = meshShapeScaled->GetSize();
 
   // Note: dartsim uses assimp for storing mesh data, and assimp by default uses
@@ -234,14 +236,14 @@ TEST(EntityManagement_TEST, ConstructEmptyWorld)
 
 TEST(EntityManagement_TEST, RemoveEntities)
 {
-  ignition::plugin::Loader loader;
+  plugin::Loader loader;
   loader.LoadLib(dartsim_plugin_LIB);
 
-  ignition::plugin::PluginPtr dartsim =
+  plugin::PluginPtr dartsim =
       loader.Instantiate("ignition::physics::dartsim::Plugin");
 
   auto engine =
-      ignition::physics::RequestEngine3d<TestFeatureList>::From(dartsim);
+      physics::RequestEngine3d<TestFeatureList>::From(dartsim);
   ASSERT_NE(nullptr, engine);
 
   auto world = engine->ConstructEmptyWorld("empty world");
