@@ -20,20 +20,21 @@
 #include <iostream>
 #include <set>
 
-#include <ignition/math/Vector3.hh>
-#include <ignition/math/eigen3/Conversions.hh>
+#include <gz/math/Vector3.hh>
+#include <gz/math/eigen3/Conversions.hh>
 
-#include <ignition/physics/FindFeatures.hh>
-#include <ignition/plugin/Loader.hh>
-#include <ignition/physics/RequestEngine.hh>
+#include <gz/physics/FindFeatures.hh>
+#include <gz/plugin/Loader.hh>
+#include <gz/physics/RequestEngine.hh>
 
 // Features
-#include <ignition/physics/ForwardStep.hh>
-#include <ignition/physics/FrameSemantics.hh>
-#include <ignition/physics/GetContacts.hh>
-#include <ignition/physics/GetEntities.hh>
-#include <ignition/physics/Shape.hh>
-#include <ignition/physics/sdf/ConstructWorld.hh>
+#include <gz/physics/ForwardStep.hh>
+#include <gz/physics/FrameSemantics.hh>
+#include <gz/physics/GetContacts.hh>
+#include <gz/physics/GetEntities.hh>
+#include <gz/physics/Shape.hh>
+#include <gz/physics/sdf/ConstructWorld.hh>
+#include <gz/physics/ContactProperties.hh>
 
 #include <sdf/Root.hh>
 #include <sdf/World.hh>
@@ -41,43 +42,54 @@
 #include <test/PhysicsPluginsList.hh>
 #include <test/Utils.hh>
 
-struct TestFeatureList : ignition::physics::FeatureList<
-    ignition::physics::LinkFrameSemantics,
-    ignition::physics::ForwardStep,
-    ignition::physics::GetContactsFromLastStepFeature,
-    ignition::physics::GetEntities,
-    ignition::physics::GetShapeBoundingBox,
-    ignition::physics::CollisionFilterMaskFeature,
-    ignition::physics::sdf::ConstructSdfWorld
+using namespace ignition;
+
+struct TestFeatureList : physics::FeatureList<
+    physics::LinkFrameSemantics,
+    physics::ForwardStep,
+    physics::GetContactsFromLastStepFeature,
+    physics::GetEntities,
+    physics::GetShapeBoundingBox,
+    physics::CollisionFilterMaskFeature,
+#ifdef DART_HAS_CONTACT_SURFACE
+    physics::SetContactPropertiesCallbackFeature,
+#endif
+    physics::sdf::ConstructSdfWorld
 > { };
 
-using TestWorldPtr = ignition::physics::World3dPtr<TestFeatureList>;
-using TestShapePtr = ignition::physics::Shape3dPtr<TestFeatureList>;
-using ContactPoint = ignition::physics::World3d<TestFeatureList>::ContactPoint;
-using ExtraContactData =
-    ignition::physics::World3d<TestFeatureList>::ExtraContactData;
+using TestWorldPtr = physics::World3dPtr<TestFeatureList>;
+using TestShapePtr = physics::Shape3dPtr<TestFeatureList>;
+using TestWorld = physics::World3d<TestFeatureList>;
+using TestContactPoint = TestWorld::ContactPoint;
+using TestExtraContactData = TestWorld::ExtraContactData;
+using TestContact = TestWorld::Contact;
+#ifdef DART_HAS_CONTACT_SURFACE
+using ContactSurfaceParams =
+  physics::SetContactPropertiesCallbackFeature::
+    ContactSurfaceParams<TestWorld::Policy>;
+#endif
 
 std::unordered_set<TestWorldPtr> LoadWorlds(
     const std::string &_library,
     const std::string &_world)
 {
-  ignition::plugin::Loader loader;
+  plugin::Loader loader;
   loader.LoadLib(_library);
 
   const std::set<std::string> pluginNames =
-      ignition::physics::FindFeatures3d<TestFeatureList>::From(loader);
+      physics::FindFeatures3d<TestFeatureList>::From(loader);
 
   EXPECT_LT(0u, pluginNames.size());
 
   std::unordered_set<TestWorldPtr> worlds;
   for (const std::string &name : pluginNames)
   {
-    ignition::plugin::PluginPtr plugin = loader.Instantiate(name);
+    plugin::PluginPtr plugin = loader.Instantiate(name);
 
     std::cout << " -- Plugin name: " << name << std::endl;
 
     auto engine =
-        ignition::physics::RequestEngine3d<TestFeatureList>::From(plugin);
+        physics::RequestEngine3d<TestFeatureList>::From(plugin);
     EXPECT_NE(nullptr, engine);
 
     sdf::Root root;
@@ -101,9 +113,9 @@ std::unordered_set<TestWorldPtr> LoadWorlds(
 bool StepWorld(const TestWorldPtr &_world, bool _firstTime,
     const std::size_t _numSteps = 1)
 {
-  ignition::physics::ForwardStep::Input input;
-  ignition::physics::ForwardStep::State state;
-  ignition::physics::ForwardStep::Output output;
+  physics::ForwardStep::Input input;
+  physics::ForwardStep::State state;
+  physics::ForwardStep::Output output;
 
   bool checkedOutput = false;
   for (size_t i = 0; i < _numSteps; ++i)
@@ -173,14 +185,14 @@ TEST_P(SimulationFeatures_TEST, ShapeBoundingBox)
     auto groundAABB =
         groundCollision->GetAxisAlignedBoundingBox(*groundCollision);
 
-    EXPECT_EQ(ignition::math::Vector3d(-1, -1, -1),
-              ignition::math::eigen3::convert(sphereAABB).Min());
-    EXPECT_EQ(ignition::math::Vector3d(1, 1, 1),
-              ignition::math::eigen3::convert(sphereAABB).Max());
-    EXPECT_EQ(ignition::math::Vector3d(-50, -50, -0.5),
-              ignition::math::eigen3::convert(groundAABB).Min());
-    EXPECT_EQ(ignition::math::Vector3d(50, 50, 0.5),
-              ignition::math::eigen3::convert(groundAABB).Max());
+    EXPECT_EQ(math::Vector3d(-1, -1, -1),
+              math::eigen3::convert(sphereAABB).Min());
+    EXPECT_EQ(math::Vector3d(1, 1, 1),
+              math::eigen3::convert(sphereAABB).Max());
+    EXPECT_EQ(math::Vector3d(-50, -50, -0.5),
+              math::eigen3::convert(groundAABB).Min());
+    EXPECT_EQ(math::Vector3d(50, 50, 0.5),
+              math::eigen3::convert(groundAABB).Max());
 
     // Test the bounding boxes in the world frames
     sphereAABB = sphereCollision->GetAxisAlignedBoundingBox();
@@ -191,14 +203,14 @@ TEST_P(SimulationFeatures_TEST, ShapeBoundingBox)
     // a 45-degree rotation, the dimensions that are orthogonal to the axis of
     // rotation will dilate from 1.0 to sqrt(2).
     const double d = std::sqrt(2);
-    EXPECT_EQ(ignition::math::Vector3d(-d, -1, 2.0 - d),
-              ignition::math::eigen3::convert(sphereAABB).Min());
-    EXPECT_EQ(ignition::math::Vector3d(d, 1, 2 + d),
-              ignition::math::eigen3::convert(sphereAABB).Max());
-    EXPECT_EQ(ignition::math::Vector3d(-50*d, -50*d, -1),
-              ignition::math::eigen3::convert(groundAABB).Min());
-    EXPECT_EQ(ignition::math::Vector3d(50*d, 50*d, 0),
-              ignition::math::eigen3::convert(groundAABB).Max());
+    EXPECT_EQ(math::Vector3d(-d, -1, 2.0 - d),
+              math::eigen3::convert(sphereAABB).Min());
+    EXPECT_EQ(math::Vector3d(d, 1, 2 + d),
+              math::eigen3::convert(sphereAABB).Max());
+    EXPECT_EQ(math::Vector3d(-50*d, -50*d, -1),
+              math::eigen3::convert(groundAABB).Min());
+    EXPECT_EQ(math::Vector3d(50*d, 50*d, 0),
+              math::eigen3::convert(groundAABB).Max());
   }
 }
 
@@ -262,19 +274,6 @@ TEST_P(SimulationFeatures_TEST, RetrieveContacts)
     auto groundPlane = world->GetModel("ground_plane");
     auto groundPlaneCollision = groundPlane->GetLink(0)->GetShape(0);
 
-    // The first step already has contacts, but the contact force due to the
-    // impact does not match the steady-state force generated by the
-    // body's weight.
-    auto checkedOutput = StepWorld(world, true);
-    EXPECT_TRUE(checkedOutput);
-
-    // After a second step, the contact force reaches steady-state
-    checkedOutput = StepWorld(world, false);
-    EXPECT_FALSE(checkedOutput);
-
-    auto contacts = world->GetContactsFromLastStep();
-    EXPECT_EQ(4u, contacts.size());
-
     // Use a set because the order of collisions is not determined.
     std::set<TestShapePtr> possibleCollisions = {
         groundPlaneCollision,
@@ -301,9 +300,14 @@ TEST_P(SimulationFeatures_TEST, RetrieveContacts)
       {sphere->GetLink(3)->GetShape(0), 3.0 * gravity},
     };
 
-    for (auto &contact : contacts)
+    // This procedure checks the validity of a generated contact point. It is
+    // used both when checking the contacts after the step is finished and for
+    // checking them inside the contact joint properties callback. The callback
+    // is called after the contacts are generated but before they affect the
+    // physics. That is why contact force is zero during the callback.
+    auto checkContact = [&](const TestContact& _contact, const bool zeroForce)
     {
-      const auto &contactPoint = contact.Get<ContactPoint>();
+      const auto &contactPoint = _contact.Get<TestContactPoint>();
       ASSERT_TRUE(contactPoint.collision1);
       ASSERT_TRUE(contactPoint.collision2);
 
@@ -325,11 +329,11 @@ TEST_P(SimulationFeatures_TEST, RetrieveContacts)
 
       expectedContactPos = expectations.at(testCollision);
 
-      EXPECT_TRUE(ignition::physics::test::Equal(expectedContactPos,
+      EXPECT_TRUE(physics::test::Equal(expectedContactPos,
                                                  contactPoint.point, 1e-6));
 
       // Check if the engine populated the extra contact data struct
-      const auto* extraContactData = contact.Query<ExtraContactData>();
+      const auto* extraContactData = _contact.Query<TestExtraContactData>();
       ASSERT_NE(nullptr, extraContactData);
 
       // The normal of the contact force is a vector pointing up (z positive)
@@ -342,13 +346,144 @@ TEST_P(SimulationFeatures_TEST, RetrieveContacts)
       EXPECT_NEAR(extraContactData->force[0], 0.0, 1e-3);
       EXPECT_NEAR(extraContactData->force[1], 0.0, 1e-3);
       EXPECT_NEAR(extraContactData->force[2],
-                  forceExpectations.at(testCollision), 1e-3);
+                  zeroForce ? 0 : forceExpectations.at(testCollision), 1e-3);
+    };
+
+#ifdef DART_HAS_CONTACT_SURFACE
+    size_t numContactCallbackCalls = 0u;
+    auto contactCallback = [&](
+      const TestContact& _contact,
+      size_t _numContactsOnCollision,
+      ContactSurfaceParams& _surfaceParams)
+    {
+      numContactCallbackCalls++;
+      checkContact(_contact, true);
+      EXPECT_EQ(1u, _numContactsOnCollision);
+      // the values in _surfaceParams are implemented as std::optional to allow
+      // physics engines fill only those parameters that are actually
+      // implemented
+      ASSERT_TRUE(_surfaceParams.frictionCoeff.has_value());
+      ASSERT_TRUE(_surfaceParams.secondaryFrictionCoeff.has_value());
+      // not implemented in DART yet
+      EXPECT_FALSE(_surfaceParams.rollingFrictionCoeff.has_value());
+      // not implemented in DART yet
+      EXPECT_FALSE(_surfaceParams.secondaryRollingFrictionCoeff.has_value());
+      // not implemented in DART yet
+      EXPECT_FALSE(_surfaceParams.torsionalFrictionCoeff.has_value());
+      ASSERT_TRUE(_surfaceParams.slipCompliance.has_value());
+      ASSERT_TRUE(_surfaceParams.secondarySlipCompliance.has_value());
+      ASSERT_TRUE(_surfaceParams.restitutionCoeff.has_value());
+      ASSERT_TRUE(_surfaceParams.firstFrictionalDirection.has_value());
+      ASSERT_TRUE(_surfaceParams.contactSurfaceMotionVelocity.has_value());
+      // these constraint parameters are implemented in DART but are not filled
+      // when the callback is called; they are only read after the callback ends
+      EXPECT_FALSE(_surfaceParams.errorReductionParameter.has_value());
+      EXPECT_FALSE(_surfaceParams.maxErrorReductionVelocity.has_value());
+      EXPECT_FALSE(_surfaceParams.maxErrorAllowance.has_value());
+      EXPECT_FALSE(_surfaceParams.constraintForceMixing.has_value());
+
+      EXPECT_NEAR(_surfaceParams.frictionCoeff.value(), 1.0, 1e-6);
+      EXPECT_NEAR(_surfaceParams.secondaryFrictionCoeff.value(), 1.0, 1e-6);
+      EXPECT_NEAR(_surfaceParams.slipCompliance.value(), 0.0, 1e-6);
+      EXPECT_NEAR(_surfaceParams.secondarySlipCompliance.value(), 0.0, 1e-6);
+      EXPECT_NEAR(_surfaceParams.restitutionCoeff.value(), 0.0, 1e-6);
+
+      EXPECT_TRUE(physics::test::Equal(Eigen::Vector3d(0, 0, 1),
+        _surfaceParams.firstFrictionalDirection.value(), 1e-6));
+
+      EXPECT_TRUE(physics::test::Equal(Eigen::Vector3d(0, 0, 0),
+        _surfaceParams.contactSurfaceMotionVelocity.value(), 1e-6));
+    };
+    world->AddContactPropertiesCallback("test", contactCallback);
+#endif
+
+    // The first step already has contacts, but the contact force due to the
+    // impact does not match the steady-state force generated by the
+    // body's weight.
+    StepWorld(world, true);
+
+#ifdef DART_HAS_CONTACT_SURFACE
+    // There are 4 collision bodies in the world all colliding at the same time
+    EXPECT_EQ(4u, numContactCallbackCalls);
+#endif
+
+    // After a second step, the contact force reaches steady-state
+    StepWorld(world, false);
+
+#ifdef DART_HAS_CONTACT_SURFACE
+    // There are 4 collision bodies in the world all colliding at the same time
+    EXPECT_EQ(8u, numContactCallbackCalls);
+#endif
+
+    auto contacts = world->GetContactsFromLastStep();
+    EXPECT_EQ(4u, contacts.size());
+
+    for (auto &contact : contacts)
+    {
+      checkContact(contact, false);
     }
+
+#ifdef DART_HAS_CONTACT_SURFACE
+    // removing a non-existing callback yields no error but returns false
+    EXPECT_FALSE(world->RemoveContactPropertiesCallback("foo"));
+
+    // removing an existing callback works and the callback is no longer called
+    EXPECT_TRUE(world->RemoveContactPropertiesCallback("test"));
+
+    // Third step
+    StepWorld(world, false);
+
+    // Number of callback calls is the same as after the 2nd call
+    EXPECT_EQ(8u, numContactCallbackCalls);
+
+    // Now we check that changing _surfaceParams inside the contact properties
+    // callback affects the result of the simulation; we set
+    // contactSurfaceMotionVelocity to [1,0,0] which accelerates the contact
+    // points from 0 m/s to 1 m/s in a single simulation step.
+
+    auto contactCallback2 = [&](
+      const TestContact& /*_contact*/,
+      size_t /*_numContactsOnCollision*/,
+      ContactSurfaceParams& _surfaceParams)
+    {
+      numContactCallbackCalls++;
+      // friction direction is [0,0,1] and contact surface motion velocity uses
+      // the X value to denote the desired velocity along the friction direction
+      _surfaceParams.contactSurfaceMotionVelocity->x() = 1.0;
+    };
+    world->AddContactPropertiesCallback("test2", contactCallback2);
+
+    numContactCallbackCalls = 0u;
+    // Fourth step
+    StepWorld(world, false);
+    EXPECT_EQ(4u, numContactCallbackCalls);
+
+    // Adjust the expected forces to account for the added acceleration along Z
+    forceExpectations =
+    {
+      // Contact force expectations are:
+      // link mass * (gravity + acceleration to 1 m.s^-1 in 1 ms)
+      {sphere->GetLink(0)->GetShape(0), 0.1 * gravity + 100},
+      {sphere->GetLink(1)->GetShape(0), 1.0 * gravity + 999.99},
+      {sphere->GetLink(2)->GetShape(0), 2.0 * gravity + 1999.98},
+      {sphere->GetLink(3)->GetShape(0), 3.0 * gravity + 2999.97},
+    };
+
+    // Verify that the detected contacts correspond to the adjusted expectations
+    contacts = world->GetContactsFromLastStep();
+    EXPECT_EQ(4u, contacts.size());
+    for (auto &contact : contacts)
+    {
+      checkContact(contact, false);
+    }
+
+    EXPECT_TRUE(world->RemoveContactPropertiesCallback("test2"));
+#endif
   }
 }
 
 INSTANTIATE_TEST_CASE_P(PhysicsPlugins, SimulationFeatures_TEST,
-    ::testing::ValuesIn(ignition::physics::test::g_PhysicsPluginLibraries),); // NOLINT
+    ::testing::ValuesIn(physics::test::g_PhysicsPluginLibraries),); // NOLINT
 
 int main(int argc, char *argv[])
 {
