@@ -18,10 +18,10 @@
 #include <set>
 #include <string>
 
-#include <ignition/common/Profiler.hh>
+#include <gz/common/Profiler.hh>
 
-#include <ignition/math/Pose3.hh>
-#include <ignition/math/Vector3.hh>
+#include <gz/math/Pose3.hh>
+#include <gz/math/Vector3.hh>
 
 #include "Link.hh"
 #include "Model.hh"
@@ -42,7 +42,7 @@ class ignition::physics::tpelib::ModelPrivate
   public: std::vector<std::size_t> nestedModelIds;
 };
 
-using namespace ignition;
+using namespace gz;
 using namespace physics;
 using namespace tpelib;
 
@@ -70,7 +70,7 @@ Entity &Model::AddLink()
 {
   std::size_t linkId = Entity::GetNextId();
 
-  if (this->GetChildren().empty())
+  if (this->GetLinkCount() == 0)
   {
     this->dataPtr->firstLinkId = linkId;
     this->dataPtr->canonicalLinkId = linkId;
