@@ -18,11 +18,13 @@
 #include "SDFFeatures.hh"
 
 #include <sdf/Box.hh>
+#include <sdf/Capsule.hh>
 #include <sdf/Cylinder.hh>
+#include <sdf/Ellipsoid.hh>
 #include <sdf/Sphere.hh>
 #include <sdf/Geometry.hh>
 #include <sdf/World.hh>
-#include <ignition/common/Console.hh>
+#include <gz/common/Console.hh>
 
 namespace ignition {
 namespace physics {
@@ -282,6 +284,14 @@ Identity SDFFeatures::ConstructSdfCollision(
     shape.SetSize(boxSdf->Size());
     collision->SetShape(shape);
   }
+  else if (geom->Type() == ::sdf::GeometryType::CAPSULE)
+  {
+    const auto capsuleSdf = geom->CapsuleShape();
+    tpelib::CapsuleShape shape;
+    shape.SetRadius(capsuleSdf->Radius());
+    shape.SetLength(capsuleSdf->Length());
+    collision->SetShape(shape);
+  }
   else if (geom->Type() == ::sdf::GeometryType::CYLINDER)
   {
     const auto cylinderSdf = geom->CylinderShape();
@@ -290,6 +300,13 @@ Identity SDFFeatures::ConstructSdfCollision(
     shape.SetLength(cylinderSdf->Length());
     collision->SetShape(shape);
   }
+  else if (geom->Type() == ::sdf::GeometryType::ELLIPSOID)
+  {
+    const auto ellipsoidSdf = geom->EllipsoidShape();
+    tpelib::EllipsoidShape shape;
+    shape.SetRadii(ellipsoidSdf->Radii());
+    collision->SetShape(shape);
+  }
   else if (geom->Type() == ::sdf::GeometryType::SPHERE)
   {
     const auto sphereSdf = geom->SphereShape();
